@@ -4,7 +4,6 @@ This demonstrates using pydantic_graph for complex control flow.
 """
 
 from dataclasses import dataclass, field
-from types import NoneType
 
 from pydantic import BaseModel
 from pydantic_ai import Agent, ModelMessage, format_as_xml
@@ -14,10 +13,48 @@ from pydantic_graph import (
     Graph,
     GraphRunContext,
 )
-from pixie import pixie_app, PixieGenerator, UserInputRequirement
+import pixie.sdk as pixie
 
-# Agent for asking questions
-ask_agent = Agent("openai:gpt-4o-mini", output_type=str)
+
+class EvaluationOutput(BaseModel, use_attribute_docstrings=True):
+    correct: bool
+    """Whether the answer is correct."""
+    comment: str
+    """Comment on the answer, reprimand the user if the answer is wrong."""
+
+
+ask_agent_prompt = pixie.create_prompt(
+    "question_ask_agent",
+    description="Asks simple questions with a single correct answer",
+)
+evaluate_agent_prompt = pixie.create_prompt(
+    "question_evaluate_agent",
+    description="Evaluates if an answer to a question is correct",
+)
+
+# Agents will be initialized lazily inside the app handler
+ask_agent: Agent
+evaluate_agent: Agent[None, EvaluationOutput]
+
+
+def get_ask_agent() -> Agent:
+    global ask_agent
+    if ask_agent is None:
+        ask_agent = Agent(
+            "openai:gpt-4o-mini", system_prompt=ask_agent_prompt.compile()
+        )
+    return ask_agent
+
+
+def get_evaluate_agent() -> Agent[None, EvaluationOutput]:
+    global evaluate_agent
+    if evaluate_agent is None:
+        evaluate_agent = Agent(
+            "openai:gpt-4o-mini",
+            output_type=EvaluationOutput,
+            system_prompt=evaluate_agent_prompt.compile(),
+        )
+    return evaluate_agent
 
 
 @dataclass
@@ -33,7 +70,7 @@ class Ask(BaseNode[QuestionState]):
     """Generate a question using the AI."""
 
     async def run(self, ctx: GraphRunContext[QuestionState]) -> "Answer":
-        agent_result = await ask_agent.run(
+        agent_result = await get_ask_agent().run(
             "Ask a simple question with a single correct answer.",
             message_history=ctx.state.ask_agent_messages,
         )
@@ -55,21 +92,6 @@ class Answer(BaseNode[QuestionState]):
         return Evaluate(ctx.state.answer)
 
 
-class EvaluationOutput(BaseModel, use_attribute_docstrings=True):
-    correct: bool
-    """Whether the answer is correct."""
-    comment: str
-    """Comment on the answer, reprimand the user if the answer is wrong."""
-
-
-# Agent for evaluating answers
-evaluate_agent = Agent(
-    "openai:gpt-4o-mini",
-    output_type=EvaluationOutput,
-    system_prompt="Given a question and answer, evaluate if the answer is correct.",
-)
-
-
 @dataclass
 class Evaluate(BaseNode[QuestionState, None, str]):
     """Evaluate the user's answer."""
@@ -81,7 +103,7 @@ class Evaluate(BaseNode[QuestionState, None, str]):
         ctx: GraphRunContext[QuestionState],
     ) -> End[str] | "Reprimand":
         assert ctx.state.question is not None
-        agent_result = await evaluate_agent.run(
+        agent_result = await get_evaluate_agent().run(
             format_as_xml({"question": ctx.state.question, "answer": self.answer}),
             message_history=ctx.state.evaluate_agent_messages,
         )
@@ -109,14 +131,13 @@ _question_graph = Graph(
 )
 
 
-@pixie_app
-async def question_graph(_: NoneType) -> PixieGenerator[str, str]:
+@pixie.app
+async def pydantic_ai_question_graph() -> pixie.PixieGenerator[str, str]:
     """Interactive Q&A game using graph-based state machine.
 
     The AI asks questions, the user answers, and the AI evaluates.
     If wrong, user gets reprimanded and a new question is asked.
     """
-    Agent.instrument_all()
 
     yield "🧠 Welcome to the Q&A Challenge!"
     yield "I'll ask you questions and evaluate your answers.\n"
@@ -136,7 +157,7 @@ async def question_graph(_: NoneType) -> PixieGenerator[str, str]:
                 # Ask user for their answer
                 yield f"\n❓ Question: {node.question}"
                 yield "What is your answer?"
-                state.answer = yield UserInputRequirement(str)
+                state.answer = yield pixie.InputRequired(str)
                 # Continue with the user's answer by passing it to the next iteration
                 # The graph will automatically move to Evaluate with this answer
                 continue
@@ -152,19 +173,3 @@ async def question_graph(_: NoneType) -> PixieGenerator[str, str]:
                 yield f"\n✅ {node.data}"
                 yield "\n🎉 Congratulations! You got it right!"
                 return
-
-
-# For local testing
-async def test():
-    """Test function for local development."""
-    async for message in question_graph(None):
-        if isinstance(message, UserInputRequirement):
-            # Simulate user input for testing
-            message = "Paris"  # Assuming a capital question
-        print(message)
-
-
-if __name__ == "__main__":
-    import asyncio
-
-    asyncio.run(test())
